#include "gtest/gtest.h"

#include "configuration/fee.h"
#include "enums/types.h"
#include "transactions/builder.h"
using namespace Ark::Crypto::Transactions;

TEST(transactions, build_transfer) {
  const auto actual = Ark::Crypto::Transactions::Builder::buildTransfer("D61mfSggzbvQgTUe6JhYKH2doHaqJ3Dyib", 100000000,
                                                                        "", "Secret passphrase");
  ASSERT_EQ(Ark::Crypto::Enums::Types::TRANSFER, actual.type);
  ASSERT_TRUE(Ark::Crypto::Configuration::Fee().get(Ark::Crypto::Enums::Types::TRANSFER) == actual.fee);
  ASSERT_STREQ("D61mfSggzbvQgTUe6JhYKH2doHaqJ3Dyib", actual.recipientId.c_str());
  ASSERT_TRUE(100000000 == actual.amount);
  ASSERT_STREQ("", actual.vendorField.c_str());

  // test 0 ARKtoshi value
  const auto shouldBeEmpty = Ark::Crypto::Transactions::Builder::buildTransfer(
      "D61mfSggzbvQgTUe6JhYKH2doHaqJ3Dyib",
      0,
      "",
      "Secret passphrase");
  ASSERT_STREQ("", shouldBeEmpty.recipientId.c_str());
  ASSERT_EQ(0, shouldBeEmpty.amount);
  ASSERT_FALSE(shouldBeEmpty.verify());
}
